{***********************************************************}
{   This file is part of the DORB project.                  }
{                                                           }
{   See the file License, included in this distribution,    }
{   for details about the copyright.                        }
{                                                           }
{   Send comments and/or bug reports to: vic@inec.ru        }
{***********************************************************}
unit idldb;

interface

uses
  classes,parser,ir_int,ir,sysutils;

type

   TDB = class
   private
     FRepoidFilesnames : TStringList;
     FPrefixes : TStringList;
     FVersions : TStringList;
     FVersionNames : TStringList;
     FIDs : TStringList;
     FIDNames : TStringList;
     FScopeNames : TStringList;
   public
     constructor Create();
     destructor Destroy(); override;
     procedure register_repoid(node: TNode; id: string);
     function repoid_defined(id: string): Boolean;
     function repoid_file_name(id: string): string;
     procedure register_anonymous(node: TNode; t : IIDLType);
     function pseudo_repoid(t: IIDLType): string;
     procedure enter_scope(name: string);
     procedure leave_scope();
     procedure set_prefix(val: string);
     function get_prefix: string;
     function get_id_prefix: string;
     procedure set_version(name: string; val: string);
     procedure enter_include();
     procedure leave_include();
     procedure set_repoids(rep: IRepository);
     procedure set_repo_id(name,id: string);
   end;

implementation

{ TDB }
uses
  orb_int;

constructor TDB.Create;
begin
  FRepoidFilesnames := TStringList.Create;
  FPrefixes := TStringList.Create;
  FVersions := TStringList.Create;
  FVersionNames := TStringList.Create;
  FIDs := TStringList.Create;
  FIDNames := TStringList.Create;
  FScopeNames := TStringList.Create;
  FPrefixes.Add('');
end;

destructor TDB.Destroy;
begin
  FRepoidFilesnames.free;
  FVersions.Free;
  FVersionNames.Free;
  FIDs.Free;
  FIDNames.Free;
  FPrefixes.Free;
  FScopeNames.Free;
  inherited;
end;

procedure TDB.enter_include;
begin
  FScopeNames.Add('');
  FPrefixes.Add('');
end;

procedure TDB.enter_scope(name: string);
begin
  FScopeNames.Add(name);
  FPrefixes.Add(FPrefixes[FPrefixes.count-1]);
end;

function TDB.get_id_prefix: string;
var
  i : integer;
begin
  result := FPrefixes[FPrefixes.Count-1];
  for i := 0 to pred(FScopeNames.count) do
    begin
      if result <> '' then result := result + '_';
      result := result + FScopeNames[i];
    end;
  if result <> '' then result := result + '_';
end;

function TDB.get_prefix: string;
var
  i : integer;
begin
  result := '';
  for i := pred(FScopeNames.count) downto 0 do
    begin
      if FScopeNames[i] = '' then break;
      result := FScopeNames[i] + '/' + result;
    end;
  if FPrefixes[FPrefixes.Count-1] <> '' then
    result := FPrefixes[FPrefixes.Count-1] + '/' + result;
end;

procedure TDB.leave_include;
begin
  FScopeNames.Delete(FScopeNames.Count-1);
  FPrefixes.Delete(FPrefixes.Count-1);
end;

procedure TDB.leave_scope;
begin
  FScopeNames.Delete(FScopeNames.Count-1);
  FPrefixes.Delete(FPrefixes.Count-1);
end;

function TDB.pseudo_repoid(t: IIDLType): string;
var
  con: IContained;
  seq: ISequenceDef;
  arr : IArrayDef;
  pr : IPrimitiveDef;
begin
  result := '';
  con := TContained._narrow(t as IORBObject);
  if con <> nil then
    begin
      result := con.id;
      exit;
    end;
  seq := TSequenceDef._narrow(t as IORBObject);
  if seq <> nil then
    begin
      result := 'sequence_'+pseudo_repoid(seq.element_type_def)+'_'+inttostr(seq.bound) ;
      exit;
    end;
  arr := TArrayDef._narrow(t as IORBObject);
  if arr <> nil then
    begin
      result := 'array_'+pseudo_repoid(arr.element_type_def)+'_'+inttostr(arr.length) ;
      exit;
    end;
  pr := TPrimitiveDef._narrow(t as IORBObject);
  if pr <> nil then
    begin
      result := 'primitive_'+inttostr(ord(pr.kind));
      exit;
    end;
  // IFixedDef
  // IStringDef
  // IWStringDef
end;

procedure TDB.register_anonymous(node: TNode; t: IIDLType);
var
  prep : string;
begin
  if node.top_level then exit;
  prep := pseudo_repoid(t);
  FRepoidFilesnames.Values[prep] := node.file_name;
end;

procedure TDB.register_repoid(node: TNode; id: string);
begin
  if not node.top_level then
    FRepoidFilesnames.Values[id] := node.file_name;
end;

function TDB.repoid_defined(id: string): Boolean;
begin
  result := FRepoidFilesnames.Values[id] <> '';
end;

function TDB.repoid_file_name(id: string): string;
begin
  result := FRepoidFilesnames.Values[id]
end;

procedure TDB.set_prefix(val: string);
begin
  FPrefixes[FPrefixes.Count-1] := val;
end;

procedure TDB.set_repoids(rep: IRepository);
var
  i : integer;
  c: IContained;
begin
  for i := 0 to Pred(FVersions.count) do
    begin
      c := rep.lookup(FVersionNames[i]);
      if c <> nil then c.version(FVersions[i]);
    end;
  for i := 0 to Pred(FIDs.count) do
    begin
      c := rep.lookup(FIDNames[i]);
      if c <> nil then c.id(FIDs[i]);
    end;
end;

procedure TDB.set_repo_id(name, id: string);
begin
  FIDs.add(id);
  FIDNames.Add(name);
end;

procedure TDB.set_version(name, val: string);
begin
  FVersionNames.Add(name);
  FVersions.Add(val);
end;

end.
