program idltopas;
{$APPTYPE CONSOLE}
uses
  SysUtils,
  Windows,
  ir_int,
  orb_int,
  ir,
  types,
  classes,
  idl in 'idl.pas',
  parser in 'parser.pas',
  codebase in 'codebase.pas',
  scanner in 'scanner.pas',
  codeunit in 'codeunit.pas',
  codeutil in 'codeutil.pas',
  cpp in 'cpp.pas',
  idldb in 'idldb.pas',
  YaccLib in 'Yacclib.pas',
  LexLib in 'Lexlib.pas';

var
  name,str,sstr : string;
  preprocess,fname_defined: Boolean;
  omit_preprocess : Boolean;
  con: IRepository;
  params: TStrings;
  i,j,p : integer;
  db: TDB;
  ch : Char;

procedure print_usage;
begin
  writeln('usage: idltopas [<options>] [<idlfile>]');
  writeln('where <options> are: ');
  writeln('  -h, -help : print this message');
  writeln('  -poa      : generate code for POA');
  writeln('  -ixxx     : path for include files');
  writeln('  -p        : preprocess to stdout');
  writeln('  -any      : generate code for Any marshalling');
  writeln('  -np       : don`t preprocess');
  writeln('  -impl     : generate example implementation');
  Halt(1);
end;

begin
  preprocess := false;
  omit_preprocess := false;
  params := TStringList.Create;
  fname_defined := false;
  DecimalSeparator := '.';
  for i := 1 to Paramcount do
    begin
      name := lowercase(paramstr(i));
      if (name[1] = '-') and (name[2] <> '\') and (name[2] <> '/') then
        begin
          if name = '-p' then
            preprocess := true
          else if name = '-np' then
            omit_preprocess := true
          else if (name = '-poa') or (name = '-impl') or (name = '-any') then
            params.add(name)
          else if pos('-i',name) = 1 then // includes search paths
            begin
              str := copy(name,3,1000);
              p := 1;
              j := 1;
              while j < length(str) do
                begin
                  if str[j] = ';' then
                    begin
                      sstr := copy(str,p,j-1);
                      if sstr <> '' then includes.add(sstr);
                      p := j+1;
                    end;
                  inc(j);
                end;
              sstr := copy(str,p,j);
              if sstr <> '' then includes.add(sstr);
            end
          else
            begin
              writeln('unknown parameter :'+name);
              print_usage;
            end;
        end
      else if not fileexists(name) then
        begin
          writeln('file not found :'+name);
          print_usage
        end
      else
        begin
          if fname_defined then
            begin
              writeln('neme already defined :'+name);
              print_usage;
            end;
          fname_defined := true;
          params.Insert(0,name);
          filename := name;
        end;
    end;
  if not fname_defined then
    begin
      writeln('missing file name');
      print_usage;
    end;
  yyoutput := TMemoryStream.Create;
  try
    yyinput := TFileStream.Create(filename,fmOpenRead);
    if not omit_preprocess then
      begin
        try
          yyprintln('#line 1 "'+filename+'"');
          yyinput.position := 0;
          cpp.yylex;
        finally
          yyinput.free;
          yyinput := nil;
        end;
        yyoutput.Position := 0;
        if preprocess then
          begin
            while yyoutput.Position < yyoutput.Size do
              begin
                yyoutput.Read(ch,1);
                write(ch);
              end;
            exit;
          end;
        yyinput := yyoutput;
      end;
    yyoutput := THandleStream.Create(GetStdHandle(STD_OUTPUT_HANDLE));
    theParser := TParser.Create;
    try
      theParser.parse(filename);
      if theParser.getRootNode <> nil then
        try
          con:= TRepository.Create;
          db := TDB.Create;
          try
            with TScanner.Create(theParser.getRootNode,con,db) do
              try
                collect;
              finally
                free;
              end;
            with TCodegen.Create(con,params,db) do
              try
                generate;
              finally
                free;
              end;
          finally
            db.free;
          end;
        except
          on e : Exception do
            filewrite(GetStdHandle(STD_ERROR_HANDLE),PCHar(e.message)^,length(e.message));
        end;
    finally
      theParser.Free;
    end;
  finally
    yyoutput.free;
    yyinput.free;
  end;
end.
