// **********************************************************************
//
// Copyright (c) 2001 MT Tools.
//
// All Rights Reserved
//
// MT_DORB is based in part on the product DORB,
// written by Shadrin Victor
//
// See Readme.txt for contact information
//
// **********************************************************************
unit disp;

interface

{$I ..\dorb.inc}

uses
  Types,orb_int,orb,winsock,windows,classes,disp_int,sysUtils;

type

  TSelectDispatcher = class(TInterfacedObject,ISelectDispatcher,IORBDispatcher)
  private
    FCurr_rset : TFDSet;
    FCurr_wset : TFDSet;
    FCurr_xset : TFDSet;
    {**  list of event records }
    FEventList : TList;
    FLocks: integer;
    FModified : Boolean;
  private
    procedure lock();
    procedure unlock();
  protected
    procedure run(infinite: Boolean);
    procedure handle_fevents(rset,wset,xset: TFDSet);
    procedure update_fevents();
    procedure socketevent(const cb: IORBDispCallback; fd: TSocket;evt: TDispEvent);
    procedure timeevent(const cb: IORBDispCallback; timeout: longint);
    procedure move(const disp: IORBDispatcher);
    procedure remove(const cb: IORBDispCallback;evt: TDispEvent);
    function idle(): Boolean;
    procedure block(b: boolean);
    function isblocking: boolean;
  public
    constructor CreateEx();
    destructor Destroy(); override;
  end;

  PFileEvent = ^TFileEvent;
  TFileEvent = record
    EventType: TDispEvent;
    Desc : TSocket;
    Cb : IORBDispCallBack;
    Deleted : Boolean;
  end;


implementation

var
  selectDispatcherIsBlocking: boolean;

//**************************************************************
//  TSelectDispatcher
//**************************************************************
constructor TSelectDispatcher.CreateEx();
begin
  inherited Create();
  FLocks := 0;
  FModified := false;
  FEventList := TList.Create;
end;

destructor TSelectDispatcher.Destroy;
begin
  FEventList.Free;
  inherited Destroy();
end;

{**
  Selects state of specified sockets and call handle_fevents
  on success.
**}
procedure TSelectDispatcher.run(infinite: Boolean);
var
  rset,wset,xset : TFDSet;
  res : longint;
  time : TimeVal;
begin
  while true do
    begin
      rset := FCurr_rset;
      wset := FCurr_wset;
      xset := FCurr_xset;
      time.tv_sec := 1;
      time.tv_usec := 0;
      res := select(0,@rset,@wset,@xset,@time);
      Sleep(1000);
      if res > 0 then
        handle_fevents(rset,wset,xset);
      if not infinite then break;
    end;
end;

procedure TSelectDispatcher.remove(const cb: IORBDispCallback;evt: TDispEvent);
var
  i : integer;
begin
  for i := Pred(FEventList.Count) downto 0 do
    if ((PFileEvent(FEventList[i])^.EventType = evt) or (evt = _All)) and (PFileEvent(FEventList[i])^.cb = cb) then
      PFileEvent(FEventList[i])^.Deleted := true;
  update_fevents();
end;

procedure TSelectDispatcher.socketevent(const cb: IORBDispCallback; fd: TSocket; evt: TDispEvent);
var
  event : PFileEvent;
begin
  event := AllocMem(sizeof(TFileEvent));
  event^.EventType := evt;
  event^.Desc := fd;
  event^.cb := cb;
  event^.deleted := false;
  FEventList.add(event);
  update_fevents();
end;


function TSelectDispatcher.idle: Boolean;
begin
  assert(false,'not implemented');
  result := false;
end;

procedure TSelectDispatcher.move(const disp: IORBDispatcher);
begin
  assert(false,'not implemented');
end;

procedure TSelectDispatcher.timeevent(const cb: IORBDispCallback;
  timeout: longint);
begin
  assert(false,'not implemented');
end;

procedure TSelectDispatcher.update_fevents;
var
  i : integer;
begin
  FModified := true;
  FD_ZERO(FCurr_rset);
  FD_ZERO(FCurr_wset);
  FD_ZERO(FCurr_xset);
  for i := 0 to Pred(FEventList.Count) do
    with PFileEvent(FEventList[i])^ do
      if not deleted then
        case EventType of
          _Read: FD_SET(Desc,FCurr_rset);
          _Write: FD_SET(Desc,FCurr_wset);
          _Except: FD_SET(Desc,FCurr_xset);
          else
            assert(false,'not implemented');
        end;
end;

{**
  Passes trough event list and executes callback procedures for
  specified sockets
**}
procedure TSelectDispatcher.handle_fevents(rset,wset,xset: TFDSet);
var
  i : integer;
begin
  lock();
  for i := 0 to FEventList.count-1 do
    with PFileEvent(FEventList[i])^ do
      if not Deleted then
      case EventType of
        _Read : if FD_ISSET(Desc,rset) then
          CB.callback(self,_Read);
        _Write : if FD_ISSET(Desc,wset) then
          CB.callback(self,_Write);
        _Except : if FD_ISSET(Desc,xset) then
          CB.callback(self,_Except);
        else
          assert(false,'not implemented');
      end;
  unlock();
end;

procedure TSelectDispatcher.lock();
begin
  if FLocks = 0 then
    FModified := false;
  Inc(FLocks);
end;

procedure TSelectDispatcher.unlock();
var
  i : integer;
begin
  dec(FLocks);
  if (FLocks > 0) or not FModified then exit;
  for i := Pred(FEventList.Count) downto 0 do
    if PFileEvent(FEventList[i])^.Deleted then
      begin
        FreeMem(PFileEvent(FEventList[i]));
        FEventList.Delete(i);
      end;
end;

procedure TSelectDispatcher.block(b: boolean);
begin
  selectDispatcherIsBlocking := b;
end;

function TSelectDispatcher.isblocking: boolean;
begin
  result := selectDispatcherIsBlocking;
end;

initialization
{$IFDEF HAVE_THREADS}
selectDispatcherIsBlocking := true;
{$ELSE}
selectDispatcherIsBlocking := false;
{$ENDIF}
end.
