// **********************************************************************
//
// Copyright (c) 2001 MT Tools.
//
// All Rights Reserved
//
// MT_DORB is based in part on the product DORB,
// written by Shadrin Victor
//
// See Readme.txt for contact information
//
// **********************************************************************
{** Implementation of event channel specification }
unit event;

interface

uses
  orb_int,event_int,orb,code_int,imr,imr_int,req_int,types,
  classes,env_int;

type

  // stubs

  TPushConsumer_stub = class(TORBObject,IPushConsumer)
  protected
    procedure push(const data: IAny);
    procedure disconnect_push_consumer();
  end;

  TProxyPushConsumer_stub = class(TPushConsumer_stub,IPushConsumer,IProxyPushConsumer)
  protected
    procedure connect_push_supplier(const sup: IPushSupplier);
  end;

  TPushSupplier_stub = class(TORBObject,IPushSupplier)
  protected
    procedure disconnect_push_supplier();
  end;

  TProxyPushSupplier_stub = class(TPushSupplier_stub,IPushSupplier,IProxyPushSupplier)
  protected
    procedure connect_push_consumer(const cons: IPushConsumer);
  end;

  TPullSupplier_stub = class(TORBObject,IPullSupplier)
  protected
    function pull(): IAny;
    function try_pull(out has_event: Boolean): IAny;
    procedure disconnect_pull_supplier();
  end;

  TProxyPullSupplier_stub = class(TPullSupplier_stub,IPullSupplier,IProxyPullSupplier)
  protected
    procedure connect_pull_consumer(const cons: IPullConsumer);
  end;

  // skeletons
  TPushConsumer_skel = class(TImplementationBase,IPushConsumer,IStaticMethodDispatcher,IStaticInterfaceDispatcher)
  private
    FDispatchers : IInterfaceList;
  protected
    function _dispatch(const req: IStaticServerRequest; const env: IEnvironment): Boolean;
    procedure push(const data: IAny); virtual; abstract;
    procedure register_dispatcher(const disp: IStaticInterfaceDispatcher);
    procedure invoke(const req: IStaticServerRequest; const env: IEnvironment);
    function make_request(const req: IORBRequest;const obj: IORBObject; mid: ORBMessageID;
       const oa: IObjectAdaptor; pr: IPrincipal):IServerRequestBase; override;
    procedure doinvoke(const req: IServerRequestBase; const env: IEnvironment); override;
    procedure disconnect_push_consumer();
  public
    constructor Create();
    destructor Destroy(); override;
  end;

implementation

uses
  static,stdstat,sysutils,any,throw,exceptions;

//***********************************************************************
//  TPushConsumer_stub
//***********************************************************************
procedure TPushConsumer_stub.push(const data: IAny);
var
  inp : IStaticAny;
  req: IStaticRequest;
begin
  req := TStaticRequest.Create(self,'push');
  inp := TStaticAny.Create(stat_Any,@data);
  req.add_in_arg(inp);
  req.Invoke;
  dorb_static_throw(req);
end;

procedure TPushConsumer_stub.disconnect_push_consumer();
var
  req: IStaticRequest;
begin
  req := TStaticRequest.Create(self,'disconnect_push_consumer');
  req.invoke();
  dorb_static_throw(req);
end;

//*********************************************************************
//   TProxyPushConsumer_stub
//*********************************************************************
procedure TProxyPushConsumer_stub.connect_push_supplier(const sup: IPushSupplier);
var
  inp : IStaticAny;
  req: IStaticRequest;
begin
  req := TStaticRequest.Create(self,'connect_push_suplier');
  inp := TStaticAny.Create(stat_Object,@sup);
  req.add_in_arg(inp);
  req.Invoke;
  dorb_static_throw(req);
end;

//*********************************************************************
//   TPushSupplier_stub
//*********************************************************************
procedure TPushSupplier_stub.disconnect_push_supplier();
var
  req: IStaticRequest;
begin
  req := TStaticRequest.Create(self,'disconnect_push_supplier');
  req.invoke();
  dorb_static_throw(req);
end;

//***********************************************************************
//  TProxyPushSupplier_stub
//***********************************************************************
procedure TProxyPushSupplier_stub.connect_push_consumer(const cons: IPushConsumer);
var
  inp : IStaticAny;
  req: IStaticRequest;
begin
  req := TStaticRequest.Create(self,'connect_push_consumer');
  inp := TStaticAny.Create(stat_Object,@cons);
  req.add_in_arg(inp);
  req.Invoke;
  dorb_static_throw(req);
end;

//***************************************************************************
// TPushConsumer_skel }
//***************************************************************************
constructor TPushConsumer_skel.Create;
var
  def: IImplementationDef;
begin
  inherited Create();
  FDispatchers := TInterfaceList.Create;
  def := find_impl('IDL:omg.org/CosEventComm/PushConsumer:1.0','PushConsumer');
  create_ref('',nil,def,'IDL:omg.org/CosEventComm/PushConsumer:1.0');
  register_dispatcher(self);
end;

destructor TPushConsumer_skel.Destroy();
begin
  FDispatchers := nil;
  inherited Destroy();
end;

procedure TPushConsumer_skel.disconnect_push_consumer();
begin
  // nothing to do
end;

procedure TPushConsumer_skel.register_dispatcher(const disp: IStaticInterfaceDispatcher);
begin
  FDispatchers.Add(disp);
end;

procedure TPushConsumer_skel.invoke(const req: IStaticServerRequest; const env: IEnvironment);
begin
  // temporary
  _dispatch(req, env);
end;

function TPushConsumer_skel.make_request(const req: IORBRequest;const obj: IORBObject; mid: ORBMessageID;
  const oa: IObjectAdaptor; pr: IPrincipal):IServerRequestBase;
begin
  result := TStaticServerRequest.Create(req,obj,mid,oa,pr);
end;

procedure TPushConsumer_skel.doinvoke(const req: IServerRequestBase; const env: IEnvironment);
begin
  self.invoke(IStaticServerRequest(req),env);
end;

function TPushConsumer_skel._dispatch(const req: IStaticServerRequest; const env: IEnvironment): Boolean;
var
  a : IAny;
  data : IStaticAny;
begin
  result := false;
  try
    if req.op_name = 'push' then
      begin
        a := TAny.Create;
        data := TStaticAny.Create(stat_any,@a);
        req.add_in_arg(data);
        if not req.read_args then
          begin
            result := true;
            exit;
          end;
        try
          push(a);
        except
          req.set_exception(UserException.Create());
          req.write_results;
          result := true;
          exit;
        end;
        req.write_results();
        result := true;
        exit;
      end;
    if req.op_name = 'disconnect_push_consumer' then
      begin
        if not req.read_args then
          begin
            result := true;
            exit;
          end;
        disconnect_push_consumer();
        req.write_results;
        result := true;
        exit;
      end
  except
    req.set_exception(SystemException.Create());
    req.write_results;
    result := true;
  end;
end;

//*************************************************
//  TPullSupplier_stub
//*************************************************
procedure TPullSupplier_stub.disconnect_pull_supplier;
var
  req: IStaticRequest;
begin
  req := TStaticRequest.Create(self,'disconnect_pull_supplier');
  req.invoke();
  dorb_static_throw(req);
end;

function TPullSupplier_stub.pull: IAny;
var
  res : IStaticAny;
  req: IStaticRequest;
begin
  res := TStaticAny.Create(stat_Any);
  req := TStaticRequest.Create(self,'pull');
  req.set_result(res);
  req.invoke();
  dorb_static_throw(req);
  result := IAny(res.value^);
end;

function TPullSupplier_stub.try_pull(out has_event: Boolean): IAny;
begin
  assert(false,'not imlplemented');
end;

//*************************************************
//  TProxyPullSupplier_stub
//*************************************************
procedure TProxyPullSupplier_stub.connect_pull_consumer(
  const cons: IPullConsumer);
var
  inp : IStaticAny;
  req: IStaticRequest;
begin
  req := TStaticRequest.Create(self,'connect_pull_consumer');
  inp := TStaticAny.Create(stat_Object,@cons);
  req.add_in_arg(inp);
  req.Invoke;
  dorb_static_throw(req);
end;

end.
